/**
 * Editing functionality for Tutorial Builder tutorials
 **/

//
// Global vars
//
var currentEditableStepTextDiv = null; // step text div being edited (with old contents before being replaced with edited content)
var currentStepTextEditBoxDiv = null;   // edit box div for editing step text

var currentEditableTextOnlyDiv = null;   // text-only div (title, author, description-text) being edited (with old contets before being replaced with edited content)
var currentTextOnlyEditBoxDiv = null; // edit box div for editing text-only div

var selectedImage = null;
var hoveredToolbarButton = null;
var triggersVisible = false;

//
// Workaround: AIR html view does not render tooltips correctly
// so we are doing this manually with the code below
//
function onEditBoxToolbarButtonMouseOver(button) {
    hoveredToolbarButton = button;
    setTimeout( function() {
	if (!hoveredToolbarButton)
	    return;
	var top = hoveredToolbarButton.offsetTop+20;
	var left = hoveredToolbarButton.offsetLeft+20;
	var tooltipDiv = $('<div class="tooltip" style="position:absolute;font-size:7pt;background-color:#FFFFAA;top:' + top + 'px;left:' + left + 'px;">' + hoveredToolbarButton.title + '</div>');
	$("body").append(tooltipDiv);
    }, 1000);
}

function onEditBoxToolbarButtonMouseOut(button) {
    hoveredToolbarButton = null;
    $(".tooltip").remove();
}


//
// Create step text edit box div
//
function createStepTextEditBoxDiv() {
    var editBoxDiv = $('<div id="editbox-stepText" class="stepText"></div>');
    editBoxDiv.css("width", "500px");

    var editBoxTextArea = $('<textarea id="editbox-textarea"/>');

    var editBoxTriggers = $('<div id="editbox-triggers"/>');
    editBoxTriggers.append($('<span style="position:relative;top:-5px;">History: </span>'));
    editBoxTriggers.append($('<textarea id="history-trigger-textarea" class="trigger-textarea"/>'));    
    editBoxTriggers.append($('<span style="position:relative;top:-5px;margin-left:10px;">Tool: </span>'));
    editBoxTriggers.append($('<textarea id="tool-trigger-textarea" class="trigger-textarea"/>'));    

    var editBoxScreenshots = $('<div id="editbox-screenshots"></div>');

    editBoxDiv.append(editBoxTextArea);
    editBoxDiv.append(editBoxTriggers);
    editBoxDiv.append(editBoxScreenshots);
    
    return editBoxDiv;
}

//
// Create text-only edit box div
//
function createTextOnlyEditBoxDiv(divClass) {
    var editBoxDiv = $('<div id="editbox-textonly" class="' + divClass + '"></div>');
    var editBoxTextArea = $('<textarea id="editbox-textarea"/>');
    
    editBoxDiv.append(editBoxTextArea);

    return editBoxDiv;
}

//
// Replace text-only div with edit box and populate
//
function replaceTextOnlyDivWithEditBox(textOnlyDiv, editBoxDiv) {
    // get size of textOnlyDiv
    var width = textOnlyDiv.width();
    var height = textOnlyDiv.height();
    var lineheight = parseInt(textOnlyDiv.css("line-height"));
    var numLines = height / lineheight;

    // replace textOnlyDiv with editBoxDiv
    textOnlyDiv.replaceWith(editBoxDiv);

    // populate editBoxDiv
    $("#editbox-textarea").val(textOnlyDiv.html());    
    
    // set size of editBoxDiv
    $("#editbox-textarea").css("width", width);
    $("#editbox-textarea").attr("rows", numLines);
}

//
// Replace step text div with edit box and populate
//
function replaceStepTextDivWithEditBox(stepTextDiv, editBoxDiv) {
    // get height of text area of step (without imgs)
    var width = stepTextDiv.width();
    var height = stepTextDiv.height();
    var stepImgsHeight = stepTextDiv.find(".screenshots-container").height();
    var lineheight = parseInt(stepTextDiv.css("line-height"));
    var numLines = (height-stepImgsHeight) / lineheight;

    // replace stepTextDiv
    stepTextDiv.replaceWith(editBoxDiv);

    // move imgs from stepText to editbox-screenshots
    var screenshots = stepTextDiv.find(".screenshots");
    screenshots.appendTo($("#editbox-screenshots"));

    // populate triggers
    var stepContainerDiv = editBoxDiv.closest(".step-container");
    if (stepContainerDiv.length == 1) {
	var historyTrigger = "";
	if (stepContainerDiv.attr("historytrigger") != undefined)
	    historyTrigger = stepContainerDiv.attr("historytrigger");
	$("#history-trigger-textarea").val(historyTrigger);
	
	var toolTrigger = "";
	if (stepContainerDiv.attr("tooltrigger") != undefined)
	    toolTrigger = stepContainerDiv.attr("tooltrigger");
	$("#tool-trigger-textarea").val(toolTrigger);
    }

    // get html from stepText (imgs have been moved to editbox, so this is just html for text)
    var stepTextDivHTML = stepTextDiv.html();
    stepTextDivHTML = stepTextDivHTML.trim();
    stepTextDivHTML = stepTextDivHTML.replace(/\s+/g, " ");

    // populate editBoxDiv
    $("#editbox-textarea").val(stepTextDivHTML);

    // set size of editBoxDiv
    $("#editbox-textarea").css("width", width);
    $("#editbox-textarea").attr("rows", numLines);

    // convert textarea to WYSIWYG jHtmlArea editor
    $("#editbox-textarea").htmlarea({
	toolbar: [
	    ["bold","italic","underline"],
	    ["orderedList", "unorderedList"],
	    [{
		css: "add-image-button", 
		text: "Add image",
		action: addImage
	    }],
	    [{
		css: "show-hide-triggers", 
		text: "Show/hide triggers",
		action: showHideTriggers
	    }],
	    [{
		css: "done-editing", 
		text: "Done editing step",
		//action: saveEditedTextForCurrentStep
		action: saveEditedText
	    }],
	    [{
		css: "cancel-editing", 
		text: "Cancel editing step",
		action: cancelEditedText
	    }],
	    
	]
    });
	
    // move done/cancel buttons over to the right of toolbar
    $(".done-editing").parent().parent().css("margin-left", "167px");

    // hide stepLink (i.e., showme button)
    if (stepContainerDiv.length == 1) {
	var stepLinkDiv = stepContainerDiv.find(".stepLink");
	stepLinkDiv.css("display", "none");
    }

    // add screenshots back to stepTextDiv so that they are there in case of canceled edit
    screenshots = $("#editbox-screenshots").find(".screenshots");
    screenshots.clone().appendTo(stepTextDiv.find(".screenshots-container"));
}

//
// Check if editing is enabled
//
function isStepEditingEnabled() {
    return (currentStepTextEditBoxDiv && currentEditableStepTextDiv);
}
function isTextOnlyEditingEnabled() {
    return (currentEditableTextOnlyDiv && currentTextOnlyEditBoxDiv);
}

//
// Enable editing for step or text-only divs
//
function enableTextEditingForCurrentStep(event) {
    enableTextEditingForStep(event, currentStep);
}

function enableTextEditingForStep(event, stepNum) {

    // get current step container div
    // return if no stepContainer found
    var stepContainerDiv = $(event.target).closest(".step-container");
    if (stepContainerDiv.length != 1)
	return;

    // return if stepContainer is parent of editbox
    // (i.e., editing for this step is already enabled)
    var editBoxStepTextDescendant = stepContainerDiv.find("#editbox-stepText");
    if (editBoxStepTextDescendant.length > 0)
	return;

    var stepTextDiv = stepContainerDiv.find(".stepText");
    if (stepTextDiv.length == 1) {
	// save any pending edits
	saveEditedText();

	// create editbox
	currentStepTextEditBoxDiv = createStepTextEditBoxDiv();
	
	// replace current stepTextDiv with edit box
	replaceStepTextDivWithEditBox(stepTextDiv, currentStepTextEditBoxDiv);

	// update current editable step
	currentEditableStepTextDiv = stepTextDiv;
    }
}

function enableTextEditingForDiv(event) {

    // get current text-only div
    var textOnlyDiv = $(event.target);
    var divClass = textOnlyDiv.attr("class");
    
    // If title/author/description, replace dblclicked
    // div with new editbox div
    if ((divClass === "title") ||
	(divClass === "author") ||
	(divClass === "description-text")) {
	// save any pending edits
	saveEditedText();
	
	// create edit box
	currentTextOnlyEditBoxDiv = createTextOnlyEditBoxDiv(divClass);

	// replace current text-only div with edit box
	replaceTextOnlyDivWithEditBox(textOnlyDiv, currentTextOnlyEditBoxDiv);

	// update current editable text-only div
	currentEditableTextOnlyDiv = textOnlyDiv;
    }
}

//
// Save button updates
//
function enableSaveButton() {
    $("#save").css("background","url('images/save.png') 0 0 no-repeat");
}

function disableSaveButton() {
    $("#save").css("background","url('images/saveDisabled.png') 0 0 no-repeat");
}

//
// Save edited text
//
function saveEditedText() {
    var contentChanged = false;
    if (isStepEditingEnabled()) 
	contentChanged = saveEditedTextForCurrentStep();
    else if (isTextOnlyEditingEnabled()) 
	contentChanged = saveEditedTextForTextOnlyDiv();

    if (contentChanged)
	enableSaveButton();

    // logging
    logSaveStep(currentStep);
}

function saveEditedTextForCurrentStep() {
    if (!currentEditableStepTextDiv || !currentStepTextEditBoxDiv)
	return false;

    // get stepContainer
    var currentEditableStepContainerDiv = currentStepTextEditBoxDiv.closest(".step-container");

    // remember old stepText and triggers
    var oldStepTextHTML = currentEditableStepTextDiv.html();
    var oldHistoryTrigger = currentEditableStepContainerDiv.attr("historytrigger");
    var oldToolTrigger = currentEditableStepContainerDiv.attr("tooltrigger");

    // update textarea
    $("#editbox-textarea").htmlarea("updateTextArea");

    // put editbox step contents into editable step text div
    currentEditableStepTextDiv.html($("#editbox-textarea").val());

    // remove existing screenshots-container from current step div
    var oldScreenshots = currentEditableStepTextDiv.find(".screenshots-container");
    oldScreenshots.remove();

    // add any screenshots from editbox-screenshots
    var screenshots = $("#editbox-screenshots").find(".screenshots");
    if (screenshots.length > 0) {
	var stepDiv = currentStepTextEditBoxDiv.parent();
	var newImgsDiv = $('<div class="screenshots-container"></div>');

	for (var i=0; i<screenshots.length; i++) {
	    newImgsDiv.append(screenshots[i]);
	}

	// add new imgs div to current step div
	currentEditableStepTextDiv.append(newImgsDiv);
    }

    // update triggers
    if (currentEditableStepContainerDiv.length > 0) {
	// history trigger
	var historyTrigger = $("#history-trigger-textarea").val();
	if (historyTrigger.length > 0)
	    currentEditableStepContainerDiv.attr("historytrigger",historyTrigger);
	else
	    currentEditableStepContainerDiv.removeAttr("historytrigger");
	
	// tool trigger
	var toolTrigger = $("#tool-trigger-textarea").val();
	if (toolTrigger.length > 0)
	    currentEditableStepContainerDiv.attr("tooltrigger",toolTrigger);
	else
	    currentEditableStepContainerDiv.removeAttr("tooltrigger");
    }

    // show stepLink (i.e., showme button)
    $(".stepLink").css("display", "block");

    // check if content in stepTextDiv or triggers has changed
    var contentChanged = (oldStepTextHTML !== currentEditableStepTextDiv.html()) || (oldHistoryTrigger !== currentEditableStepContainerDiv.attr("historytrigger")) || (oldToolTrigger !== currentEditableStepContainerDiv.attr("tooltrigger"));

    // replace editbox with old step text div
    currentStepTextEditBoxDiv.replaceWith(currentEditableStepTextDiv);

    // force relayout
    setCurrentStep(currentStep);

    // reset current editing vars
    currentEditableStepTextDiv = null;
    currentStepTextEditBoxDiv = null;

    return contentChanged;
}

function saveEditedTextForTextOnlyDiv() {
    if (!currentTextOnlyEditBoxDiv || !currentEditableTextOnlyDiv)
	return false;

    // remember old text
    var oldTextHTML = currentEditableTextOnlyDiv.html();

    // update textarea
    $("#editbox-textarea").htmlarea("updateTextArea");

    // put editbox contents into text-only div
    currentEditableTextOnlyDiv.html($("#editbox-textarea").val());

    // check if content in text-only div has changed
    var contentChanged = (oldTextHTML !== currentEditableTextOnlyDiv.html());
    
    // replace editbox with old text-only div
    currentTextOnlyEditBoxDiv.replaceWith(currentEditableTextOnlyDiv);

    // rebind dblclick handler
    $("." + currentEditableTextOnlyDiv.attr("class")).bind("dblclick", enableTextEditingForDiv);

    // make sure document title matches tutorial title
    document.title = $(".title").html();
    
    // reset current editing vars
    currentEditableTextOnlyDiv = null;
    currentTextOnlyEditBoxDiv = null;

    return contentChanged;
}

//
// Cancel editing text
//
function cancelEditedText() {
    if (isStepEditingEnabled())
	cancelEditedTextForCurrentStep();
    else if (isTextOnlyEditingEnabled())
	cancelEditedTextForTextOnlyDiv();
}

function cancelEditedTextForCurrentStep() {
    if (!currentEditableStepTextDiv || !currentStepTextEditBoxDiv)
	return;

    // replace editbox with old step text div
    currentStepTextEditBoxDiv.replaceWith(currentEditableStepTextDiv);

    // show stepLink (i.e., showme button)
    $(".stepLink").css("display", "block");

    // force relayout
    setCurrentStep(currentStep);

    // reset current text vars
    currentEditableStepTextDiv = null;
    currentStepTextEditBoxDiv = null;
}

function cancelEditedTextForTextOnlyDiv() {
    if (!currentEditableTextOnlyDiv || !currentTextOnlyEditBoxDiv)
	return;

    // replace editbox with old step text div
    currentTextOnlyEditBoxDiv.replaceWith(currentEditableTextOnlyDiv);

    // force relayout
    setCurrentStep(currentStep);

    // reset current text vars
    currentEditableTextOnlyDiv = null;
    currentTextOnlyEditBoxDiv = null;
}

// 
// Delete a step
//
function deleteCurrentStep() {
    deleteStep(currentStep);
}

function deleteStep(stepNum) {
    // remove current step from dom
    var stepContainerDiv = getStep(stepNum);
    if (stepContainerDiv) {
	$(stepContainerDiv).remove();

	// update step numbers
	updateStepNumbers();

	//enable save button
	enableSaveButton();
    }
}

//
// Update step numbers to be sequential starting from 1
//
function updateStepNumbers() {
    
    $(".step-container[id]").each(function(idx, item) {

	var stepNum = idx+1;

	// update step-container id
	$(this).attr("id", "step-" + stepNum);

	// update script id
	$(this).children("script").attr("id", "tbstep-" + stepNum);

	// update stepNum
	$(this).children(".stepNum").text("Step " + stepNum);
	
	// updat showme href
	$(this).find(".showme").attr("href", "tbstep-" + stepNum);
    });

}

//
// Add image
//
function addImage() {
    // call click handler for add-image button ... this should be
    // a handler provided by container app that can access local file system
    $("#add-image").click();
}

function addImageToCurrentStep(imagePath) {
    if (!currentStepTextEditBoxDiv)
	return;

    // add image
    var screenshot = $('<img class="screenshots" onclick="imageClicked(event,this);" src="' + imagePath + '"/>');
    $("#editbox-screenshots").append(screenshot);
    
    // select image
    selectImage(screenshot);
}

//
// Remove selected image
//
function removeSelectedImage() {
    if (!selectedImage) {
	return;
    }

    selectedImage.remove();
    selectedImage = null;
}

//
// Select/unselect image
//
function imageClicked(event, imageDiv) {
    if (event.stopPropagation) {
	event.stopPropagation();
    }

    selectImage($(imageDiv));
}

function selectImage(imageJS) {

    if (imageJS) {
	unselectAllImages();

	selectedImage = imageJS

	// change outline colour of selected image
	selectedImage.css("outline", "#33BBFF solid 1.5px");
    }
}

function unselectImage(imageJS) {
    if (imageJS) {
	imageJS.css("outline", "#AAAAAA solid thin");
    }
}

function unselectAllImages() {
    $(".screenshots").css("outline", "#AAAAAA solid thin");
}

//
// Keyup on document
//
function keyUp(event) {
    if (event.keyCode === 8 || event.keyCode === 46) {
	if (isStepEditingEnabled())
	    removeSelectedImage();
	else if (!isTextOnlyEditingEnabled())
	    deleteCurrentStep();
    }
}

//
// Show/hide triggers
//
function showHideTriggers() {
    if (triggersVisible)
	hideTriggers();
    else 
	showTriggers();
}

function showTriggers() {
    // unhide triggers
    $("#editbox-triggers").css("display","block");

    triggersVisible = true;
}

function hideTriggers() {
    // hide triggers
    $("#editbox-triggers").css("display","none");
    
    triggersVisible = false;
}


//
// Get HTML to save out tutorial
//
function getHTMLToSave(outputMode) {
    // cancel editing
    //cancelEditedText();
    saveEditedText();

    // get copy of html
    var htmlCopy = $("html").clone();

    // if exporting, convert all screenshot img srcs to point at screenshots folder
    // and set the editable attribute of html tag to false
    if (outputMode === "export") {
	htmlCopy.find(".screenshots").each(function(index) {
	    var srcPath = $(this).attr("src");
	    var srcFilename = srcPath.replace(/^.*[\\\/]/, '');
	    $(this).attr("src", "screenshots/" + srcFilename);
	    //alert(index + ":" + srcFilename);
	});
	htmlCopy.attr("editable","false");

	// logging
	logExport();
    }
    else if (outputMode === "save") {
	// logging
	logSave();
    }

    // return html
    htmlCopy.find('*').removeAttr('style');
    var htmlToSave = ('<html editable="' + htmlCopy.attr("editable") + '">' + htmlCopy.html() + '</html>');
    return htmlToSave;
}

//
// Get all screenshot paths
//
function getAllScreenshotPaths() {
    var paths = [];
    $(".screenshots").each(function(index) {
	paths.push($(this).attr("src"));
    });

    //alert(paths);
    return paths;
}