//#target photoshop
//
// AnnotateClean.jsx
//
// Copyright: (c)2010, xbytor
// License: http://www.opensource.org/licenses/bsd-license.php
// Contact: xbytor@gmail.com
//

var featureXMLGlobal;

function paramsToString(params, aitem, xp, indent) {
    var pstr = '';

    if (params == undefined || params.length() == 0) {
	pstr = indent + "<No parameters>\n";

    } else {
	var childs = params.children();
	var plen = childs.length();

	for (var k = 0; k < plen; k++) {
	    var p = params.child(k);
	    var val = aitem.xpath(xp + p.lookup);
	    if (val != undefined) {
		val = val.toString();

		pstr += ("\n\t" + indent + "Name:\t" + p.name +
			 "\n\t" + indent + "XPath:\t" + p.lookup  +
			 "\n\t" + indent + "XPath(Full):\t" + xp + p.lookup +
			 "\n\t" + indent + "Value:\t" + val  + '\n');
	    }

	    var childParams = ParamsResolver.getParams(p);     // p.params;
	    pstr += paramsToString(childParams, aitem, xp + p.lookup, indent + '\t');
	}

	pstr += '\n';
    }

    return pstr;
};

function parseParams(params, aitem, node, xp) {
    var pstr = '';
    var indent = '';

    // $.level = 1; debugger;

    for (var k = 0; k < params.children().length(); k++) {
	var p = params.child(k);

	if (p.localName() == "paramsReference") {
	    ParamsResolver.resolve(params, p);
	    p = params.child(k);
	}

	var path = xp + p.lookup.toString();
	var values;
	var name = p.name.toString();

	// if (name == 'Text Style Range') {
	//   $.level = 1; debugger;
	// }

	try {
	    values = aitem.xpath(path);

	} catch (e) {
	    //$.level = 1; debugger;
	    var msg = "Bad xpath: '" + path + "'\n" + e;
	    Error.runtimeError(9010, msg);
	}

	if (values == undefined) {
	    continue;
	}

	var len = values.length();

	var parents = undefined;
	var parentPath = path.replace(/\/@?\w+$/, '');
	if (parentPath != '' && parentPath != path) {
	    try {
		parents = aitem.xpath(parentPath);
	    } catch (e) {
		var msg = "Bad xpath: '" + parentPath + "'\n" + e;
		Error.runtimeError(9010, msg);
	    }
	}

	for (var i = 0; i < len; i++) {
	    var value = values[i];

	    if (parents) {
		try {
		    var parent = parents[i];
		    if (parent && parent.@defaultSetting != undefined &&
			parent.@defaultSetting.toString() == "true") {
			// $.writeln('Removing param ' + p.name);
			continue;
		    }
		} catch (e) {
		    // $.level = 1; debugger;
		}
	    } else {
		if (value.@defaultSetting.toString() == "true") {
		    continue;
		}
	    }

	    pstr += ("\n\t" + indent + "Name:\t" + p.name +
		     "\n\t" + indent + "XPath:\t" + p.lookup  +
		     "\n\t" + indent + "XPath(Full):\t" + xp + p.lookup +
		     "\n\t" + indent + "Value:\t" + value + '\n');

	    var param = new XML('<param/>');
	    param.name = p.name;
	    param.desc = p.desc;
	    param.lookup = p.lookup;
	    param.widget = p.widget;
	    param.subUI = p.subUI;
	    param.widgettype = p.widgettype;

	    // $.level = 1; debugger;

	    var childParams = ParamsResolver.getParams(p);    // p.params;
	    if (childParams != undefined) {
		// make a copy for any child parameter (xpath scope bug)
		var scopedXML = new XML(value.toXMLString());

		// pstr += parseParams(childParams, aitem, param, xp + p.lookup);
		pstr += parseParams(childParams, scopedXML, param, '');

		// if we can't have a childless node, don't add this node
		if (param.params.length() == 0 && p.@childless.toString().toLowerCase() != "ok") {
		    // $.writeln("Childless node: " + AnnotateUI.currentId + ' in ' + AnnotateUI.currentFile);
		    continue;
		}

		if (p.value != undefined) {
		    param.value = p.value.toString();
		}

	    } else {
		var val = value.toString();

		if (p.value != undefined) {
		    // this is a hardwired value
		    val = p.value.toString();

		} else {

		    // WL: below was crashing when value has no parent
		    // added check but not sure if this is right
		    // take care of file names
		    //if (value.parent().localName() == "Alias") {
		    if (value.parent() && (value.parent().localName() == "Alias")) {
			// $.writeln("Alias: " + val);
			var parts = val.split('/');
			if (parts.length < 2) {
			    parts = val.split('\\');
			}
			val = parts.pop();
		    }
		}

		var vpath = p.valueReference.toString();
		if (vpath.startsWith('//')) {
		    val = determineValue(p, val, vpath);
		}

		param.value = val;
	    }

	    if (node.params == undefined) {
		node.params = new XML('<params/>');
	    }

	    node.params.appendChild(param);
	}
    }

    return pstr;
};

function determineValue(p, val, vpath) {
    if (vpath.contains('%')) {
	var rex = /%([^%]+)%/;
	var m;
	var str = vpath;
	while (m = rex.exec(str)) {
	    var fname = m[1];
	    var field = (fname == 'value') ? val : p[fname];
	    if (field != undefined) {
		vpath = vpath.replace(RegExp(m[0], 'g'), field.toString());
	    }
	    str = RegExp.rightContext;
	}
    }

    var res = featureXMLGlobal.xpath(vpath);
    return (res == undefined) ? val : res[0].toString();
};


function hasSameName(x1, x2) {
    return ((x1.@symname != '' && (x1.@symname == x2.@symname)) ||
	    (x1.@sym != '' && (x1.@sym == x2.@sym)));
};
function hasSameAttributes(x1, x2) {
    var attrs1 = x1.attributes();

    var len = attrs1.length();
    for (var i = 0; i < len; i++) {
	var attr = attrs1[i];
	var nm = attr.localName();
	if (nm == "sym" || nm == "symname") {
	    continue;
	}

	var v1 = attr.toString();
	var v2 = x2.elements(nm).toString();

	// var v2 = x2.attributes(nm).toString();
	if (v1 != v2) {
	    return false;
	}
    }

    return true;
};

function markDefaultSetting(xml, index) {
    // try {
    //   xml.replace(index, <default/>);
    // } catch (e) {
    //   $.level = 1; debugger;
    // }
    // return;
    xml = xml.child(index);
    xml.@defaultSetting = true;
    var list = xml.descendants();
    var len = list.length();
    for (var i = 0; i < len; i++) {
	var obj = list[i];
	obj.@defaultSetting = true;
    }
};

function trimXML(xml, refXML) {
    // $.level = 1; debugger;
    refXML = refXML.copy(); // prevents xpath scoping bug
    var children = xml.children();
    var len = children.length();
    var ochildren = refXML.children();
    var isList = xml.localName() == 'List';

    for (var i = 0; i < len; i++) {
	var child = children[i];
	var str = child.toXMLString();
	var hasChildren = child.children().length() > 0;

	if (!str) {
	    continue;
	}

	var matches;
	var ochild = undefined;
	var sameName = false;
	if (isList) {
	    try {
		ochild = ochildren[i];
		sameName = true;
		// $.level = 1; debugger;
	    } catch (e) {
	    }

	    matches = ochild;

	} else {
	    var lname = child.localName();
	    var symname = child.@symname.toString();
	    var path = path = '/' + lname + '[@symname="' + symname + '"]';
	    var matches = refXML.xpath(path);

	    // XXX In practice, we may need to encode multiple default value objects
	    // with the same symname. File Formats for File-SaveAs is primary
	    // case. This is implemented below.

	    ochild = (matches.length() > 0 ? matches[0] : undefined);

	    sameName = ochild && hasSameName(child, ochild);
	}

	if (ochild != undefined) {

	    for (var j = 0; j < matches.length(); j++) {
		// $.level = 1; debugger;
		ochild = matches[j];
		if (ochild.toXMLString() == str) {
		    // xml.replace(i, <default/>);
		    markDefaultSetting(xml, i);
		    continue;
		}
		sameName = ochild && hasSameName(child, ochild);
		if (sameName) {
		    if (hasSameAttributes(child, ochild)) {
			if (hasChildren) {
			    trimXML(child, ochild);
			} else {
			    // xml.replace(i, <default/>);
			    markDefaultSetting(xml, i);
			}
		    } else {
			trimXML(child, ochild);
		    }
		}
	    }
	}
	// if (ochild != undefined) {
	//   if (ochild.toXMLString() == str) {
	//     // xml.replace(i, <default/>);
	//     markDefaultSetting(xml, i);
	//     continue;
	//   }
	//   if (sameName) {
	//     if (hasSameAttributes(child, ochild)) {
	//       if (hasChildren) {
	//         trimXML(child, ochild);
	//       } else {
	//         // xml.replace(i, <default/>);
	//         markDefaultSetting(xml, i);
	//       }
	//     } else {
	//       trimXML(child, ochild);
	//     }
	//   }
	// }
    }
};

function isText(feature) {
    var id = feature.@id.toString();
    return ((id.contains("TypeTool") || id.contains("TypeMaskTool")) &&
	    feature.@dialog == "No" && feature.@atomicSet == "Yes");
};

function pruneDefaults(xml, defaults) {
    var pdesc = xml.ActionDescriptor;
    var desc =  defaults.ActionDescriptor;

    //$.level = 1; debugger;

    if (pdesc.toString() == desc.toString()) {
	xml.ActionDescriptor = new XML('<ActionDescriptor count="0"/>');

    } else {
	trimXML(pdesc, desc);
	// pdesc.@count = pdesc.children().length();
    }
};

function trimTextXML(xml, refXML) {
    // $.level = 1; debugger;
    refXML = refXML.copy();

    var txts = refXML.xpath('//Object[@objectTypeString="TextStyleRange"]');
    if (txts.length() > 0) {
	// we are checking for modifications to an existing text layer
	// $.level = 1; debugger;
	// $.writeln("checking existing text layer");
	trimXML(xml.Object, refXML);
	return;
    }

    // the rest of this code checks for differences
    // between a new text layer and the pre-existing type tool settings

    var textLayer = xml.Object;

    var children = textLayer.children();
    var len = children.length();

    for (var i = 0; i < len; i++) {
	var child = children[i];

	var symname = child.@symname.toString();
	var str = child.toXMLString();

	if (str) {
	    var attrs = child.attributes.toXMLString();
	    var lname = child.localName();
	    if (lname != 'List') {
		var path = '//' + lname + '[@symname="' + symname + '"]';
		var ochilds = refXML.xpath(path);
		if (ochilds.length() == 0) {
		    continue;
		}
		var ochild = ochilds[0];
		if (str == ochild.toXMLString()) {
		    // textLayer.replace(i, <default/>);
		    markDefaultSetting(textLayer, i);

		} else {
		    trimXML(child, ochild);
		}

	    } else {
		if (symname == "TextStyleRange" || symname == "paragraphStyleRange") {
		    // $.level = 1; debugger;
		    var list = child.children();
		    var llen = list.length();
		    var basename = symname.replace(/Range/, '');

		    var path = '//Object[@symname="' + basename + '"]';
		    var ochilds = refXML.xpath(path);
		    if (ochilds.length() == 0) {
			continue;
		    }
		    var ochild = ochilds[0];
		    var ocstr = ochild.toXMLString();

		    for (var j = 0; j < llen; j++) {
			var sub = list[j].Object;
			// we are only concerned with the style not the range
			if (sub.toXMLString() == ocstr) {
			    // list.replace(j, <default/>);
			    markDefaultSetting(list, j);
			} else {
			    trimXML(sub, ochild);
			}
		    }
		}
	    }
	}
    }
};

function pruneTextDefaults(xml, defaults) {
    var pdesc = xml.ActionDescriptor;
    var desc =  defaults.ActionDescriptor;

    trimTextXML(pdesc, desc);
};

function getDefaultSettings(feature, featureList) {
    // $.writeln(feature.@id);
    if (feature.defaultSettings == undefined) {
	if (feature.defaultSettingsReference != undefined) {
	    var ref = feature.defaultSettingsReference.toString();

	    if (ref) {
		delete feature.defaultSettingsReference;

		// later, handle arbitrary xpaths if needed.
		// var rPath = ref.startsWith("//") ?
		// ref : '//feature[@id="' + ref + '"]';
		var rPath = '//feature[@id="' + ref + '"]';
		var refFeature = featureList.xpath(rPath);
		if (refFeature != undefined) {
		    feature.defaultSettings = refFeature.defaultSettings.copy();

		} else {
		    var msg = "Referenced defaultSettings " + rPath + " not found!";
		    Stdlib.log(msg);
		    alert(msg);
		}
	    }
	}
    }
    return feature.defaultSettings;
};

function testXML(xml) {
    return true;
}

function Annotate_annotate(parentFolder, destFolder) {

    // Grab source Capture XML file
    var sourceXMLFilename = destFolder + '/Capture.xml';
    var sourceXML = Stdlib.readXMLFile(sourceXMLFilename);

    // WL: Debug
    var test = testXML(sourceXML);

    // Check that destination XML file location is valid
    var destXMLFilename = destFolder + '/Annotate.xml';
    var destXMLFile = new File(destXMLFilename);
    if (!destXMLFile.open("w")) {
	return self.errorPrompt("Unable to open annotate XML file for writing: " + destXMLFile.error);
    }
    destXMLFile.close();
    destXMLFilename = decodeURI(destXMLFilename);

    // Try to load biglist
    try {
	var bigListFilename = parentFolder + '/etc/BigList.xml';
	var xml = loadBigList(bigListFilename);
	
    } catch (e) {
	var msg = ("Error loading " + bigListFilename + ':  ' + e);
	alert(msg);
	Error.runtimeError(9001, msg);
    }

    featureXMLGlobal = xml;

    ParamsResolver.featureList = featureXMLGlobal;

    var list = xml.xpath('//lookup[parent::feature]');
    
    if (list == undefined || list.length() == 0) {
	alert('Features not found in XML file. Error in ' + sourceXMLFilename);
	return;
    }

    var len = list.length();

    var itemList = sourceXML.xpath('//ActionItemList');

    for (var i = 0; i < len; i++) {
	var lookup = list[i];
	var feature = lookup.parent();
	var fid = feature.@id;
	var appId = feature.@application.toString();

	if (Common.isPS() && appId == Common.PSE_APP_ID) {
	    // do not process PSE features for/in PS
	    continue;
	}

	//AnnotateUI.currentId = fid;

	try {
	    var xp = lookup.toString();

	    if (!xp) {
		continue;
	    }

	    // get the matching ActionItems
	    var res = sourceXML.xpath(xp);

	    if (res == undefined || res.length() == 0) {
		// No matches, check for dupes
		var newxp = xp.replace('ActionItem', 'Item');
		res = sourceXML.xpath(newxp);

		if (res != undefined || res.length() > 0) {
		    var fixList = new XML('<fixList/>');

		    for (var k = 0; k < res.length(); k++) {
			var item = res[k];
			var cid = item.feature.@id.toString();

			if (cid == feature.@id.toString()) {
			    // this would only occur for features with more than one
			    // lookup expression
			    continue;
			}

			if (Common.isPSE() && appId == Common.PSE_APP_ID) {
			    delete item.params;
			    fixList.appendChild(item);

			} else {
			    var msg1 = ("An ActionItem from " + sourceXML.Source +
					" matches multiple expressions including\n" +
					feature.@id.toString() + " and\n" + cid + "\n\n" +
					"Please check the log file for collisions.");
			    var msg = msg1 + "\n" + item.toXMLString() + "\n\n";
			    Stdlib.log(msg);
            
			    //               $.writeln(msg1 + "\n\n");
			    $.writeln(msg);
			    //AnnotateUI.collisions++;
			}
		    }

		    if (fixList.children().length() > 0) {
			res = fixList.children();
		    } else {
			res = undefined; // "No matches found";
		    }

		} else {
		    res = undefined; // "No matches found";
		}
	    }

	    if (res == undefined) {
		continue;
	    }

	    Stdlib.log(feature.@id.toString());

	    ParamsResolver.getParams(feature);

	    var params = feature.params;
      
	    var rstr = "\n";
	    var rlen = res.length();
      
	    for (var j = 0; j < rlen; j++) {
		var aitem = res[j];
          
		// we will not need the source later for playback
		delete aitem.src;

		//if (!opts.js) {
		//    delete aitem.literal;
		//}

		// var ixml = new XML('<Item/>');
		var ixml = aitem;

		ixml.index = ixml.childIndex();

		try {

		    // this depends on Wil's additions to BigList-ext2 <lookup> defs
		    // it's expecting <lookup name="Set">
		    // this maybe the same as aitem.@name
		    // ixml.@name = lookup.@name;  // or lookup.@event

		    ixml.lookup = xp;
		    ixml.feature = new XML('<feature/>');
		    ixml.feature.@application = feature.@application;
		    ixml.feature.@id = feature.@id;

		} catch (e) {
		    // $.level = 1; debugger;
		}

		// There appears to be a bug with xpath. aitem.xpath is not scoped to aitem.
		// Not sure what the problem is, but this bit of code fixes the problem.

		var id = feature.@id;

		// $.writeln(id);
		// continue;

		var fcopy = feature.copy();
        
		if (getDefaultSettings(fcopy, featureXMLGlobal) != undefined) {
		    if (isText(feature)) {
			pruneTextDefaults(ixml, fcopy.defaultSettings);

		    } else {
			pruneDefaults(ixml, fcopy.defaultSettings);
		    }
		}

		var scoped_aitem = new XML(aitem.toXMLString());

		// add the parameter values from the ActionItem it the item.params list
		// based on the param descriptions for this feature
		parseParams(params, scoped_aitem, ixml, ''); //, xp);
        
		aitem.setLocalName('Item');
        
		// itemList.appendChild(ixml);
        
		// the rest of this is just for logging...
		rstr += aitem.toXMLString() + '\n';
        
		var pstr = aitem.params.toXMLString();
        
		var str = ("\n====================================\n" +
			   "Feature: %s\nLookup XPath:%s\n" +
			   "Lookup:%s\nParams:%s\n").sprintf(feature.@id, xp, res, pstr);
        
		Stdlib.log(str);
	    }

	} catch (e) {
	    var msg = (Stdlib.exceptionMessage(e));
	    Stdlib.log(msg);
	    if (!confirm('Error processing XML file.\n' +
			 'feature = ' + fid + ', xp = ' + xp + '\n' +
			 'Continue?\r\r' + msg)) {
		break;
	    }
	}
    }

    var aitems = itemList.xpath("//ActionItem");
    var len = aitems.length();
    for (var i = 0; i < len; i++) {
	var aitem = aitems[i];
	var value = aitem.toXMLString().substring(0, 500);
	aitem.setLocalName('Item');
	delete aitem.src;
	//    aitem.literal = "true; // javascript code removed";

	aitem.index = aitem.childIndex();
	aitem.feature = new XML('<feature/>');
	aitem.feature.@application = "PHSP";
	aitem.feature.@id = "Unknown";
	aitem.params = new XML('<params/>');

	var param = new XML('<param/>');
	param.name = aitem.@name.toString() + ' Event';
	param.value = value;
	param.widget = "unsupported feature";

	aitem.params.appendChild(param);
    }

    var xml = new XML('<AnnotatedDemo/>');

    xml.Source = sourceXMLFilename;
    xml.Date = new Date().toString();

    itemList.setLocalName('ItemList');
    xml.appendChild(itemList);

    Stdlib.writeXMLFile(destXMLFilename, xml);
};

