#target photoshop
//
// GenerateInstructions.jsx
//

function getDescription(feature) {
  var desc = '[no description available]';

  if (feature.tbdesc != undefined && feature.tbdesc.@value.toString() != '') {
    desc = feature.tbdesc.@value.toString();
  } else if (feature.desc != undefined && feature.desc.@value.toString() != '') {
    desc = feature.desc.@value.toString();
  }

  return Common.localize(desc);
};

function createSetOpInstr(item, feature) {
  var setOpInstr = new XML('<Instruction/>');
  setOpInstr.id = feature.@id.toString();
  setOpInstr.type = "set-operation";
  // setOpInstr.desc = feature.desc.@value.toString();
  setOpInstr.desc = getDescription(feature);
  setOpInstr.name = feature.name.@value.toString();
  if (Common.isPSE() && feature.psecontext.@value.toString()) {
    setOpInstr.context = feature.psecontext.@value.toString();
  } else {
    setOpInstr.context = feature.context.@value.toString();
  }
  setOpInstr.location = feature.@type.toString();
  setOpInstr.dialog = feature.@dialog.toString();
  setOpInstr.relatedUI = feature.relatedUI;

  //setOpInstr.appendChild(item.literal);

  return setOpInstr;
}

function getNonInternalParams(param) {
	var paramXML = new XML(param.toString());
	var paramsXML = new XML('<params/>');
	var childParams = param.params.child("param");
	for (var i=0; i<childParams.length(); i++) {
		if (childParams[i].widget != "internal") {
			paramsXML.appendChild(getNonInternalParams(childParams[i]));
		}
	}
	paramXML.params = paramsXML;

	return paramXML;
}

function getParamsFromItem(item) {
  var paramsXML = new XML('<params/>');
  var childParams = item.params.child("param");
  for (var i=0; i<childParams.length(); i++) {
    if (childParams[i].widget != "internal") {
			paramsXML.appendChild(getNonInternalParams(childParams[i]));
    }
  }
  if (paramsXML.child("param").length() > 0) {
    return paramsXML;
  } else {
    return null;
  }
}

function createSetParamsInstr(item, feature) {
  var params = null;
  var params = getParamsFromItem(item);

  /*
   if (item.params.child("param").length() > 0) {
   params = item.params;
   }
   */

  if (params) {
    var setParamsInstr = new XML('<Instruction/>');
    setParamsInstr.id = feature.@id.toString();
    setParamsInstr.type = "set-params";
    // setParamsInstr.desc = feature.desc.@value.toString();
    setParamsInstr.desc = getDescription(feature);
    setParamsInstr.params = params;

    // only keep <relatedUI/in> elements that are represented in params block
    if (feature.relatedUI != undefined) {
      setParamsInstr.relatedUI = new XML('<relatedUI/>');
      var relatedUIXML = new XML(feature.relatedUI.toString());

      var outUI = relatedUIXML.xpath('//out');
      for (var i=0; i<outUI.length(); i++) {
				var newOut = new XML('<out>');
				newOut.appendChild(new XML(outUI[i].toString()));
				setParamsInstr.relatedUI.appendChild(newOut);
      }

      var inUI = relatedUIXML.xpath('//in');
			for (var i=0; i<inUI.length(); i++) {
				var newIn = new XML('<in>');
				newIn.appendChild(new XML(inUI[i].toString()));
				setParamsInstr.relatedUI.appendChild(newIn);
			}

			var initUI = relatedUIXML.xpath('//init');
			for (var i=0; i<initUI.length(); i++) {
				var newInit = new XML('<init>');
				newInit.appendChild(new XML(initUI[i].toString()));
				setParamsInstr.relatedUI.appendChild(newInit);
			}

      var subUI = setParamsInstr.params.xpath('//subUI');
			for (var i=0; i<subUI.length(); i++) {
				var newIn = new XML('<in>');
				newIn.appendChild(new XML(subUI[i].toString()));
				setParamsInstr.relatedUI.appendChild(newIn);
			}
    }

    //setParamsInstr.relatedUI = feature.relatedUI;

    //setParamsInstr.appendChild(item.literal);

    return setParamsInstr;
  } else {
    return null;
  }
}

function createCommitInstr(item, feature, prevSetParamsInstr) {
  var commitInstr = new XML('<Instruction/>');
  commitInstr.type = "commit";
  commitInstr.id = feature.@id.toString();
  commitInstr.location = feature.@type.toString();
  commitInstr.dialog = feature.@dialog.toString();
  commitInstr.event = item.@event.toString();

  // take relatedUI from prevSetParamsInstr if possible
  if (prevSetParamsInstr) {
    commitInstr.relatedUI = prevSetParamsInstr.relatedUI;
  } else {
    commitInstr.relatedUI = feature.relatedUI;
  }

  commitInstr.appendChild(item.literal);

  return commitInstr;
}

function createUnknownInstr(item, feature) {
  var unknownInstr = new XML('<Instruction/>');
  unknownInstr.id = feature.@id.toString();
  unknownInstr.type = "commit";
  unknownInstr.relatedUI = feature.relatedUI;
  var param = item.params.param[0];
  unknownInstr.name = param.name.toString();
  unknownInstr.value = param.value.toString();
  unknownInstr.literal = item.literal.toString();
  // unknownInstr.desc = feature.desc.@value.toString();
  unknownInstr.desc = getDescription(feature);
  unknownInstr.location = "None";
  unknownInstr.dialog = "No";
  return unknownInstr;
};

function GenerateInstructions_generateInstructions(parentFolder, destFolder) {

    // Grab source Annotate XML file
    var sourceXMLFilename = destFolder + '/Annotate.xml';
    var sourceXML = Stdlib.readXMLFile(sourceXMLFilename);

    // Check that destination XML file location is valid
    var destXMLFilename = destFolder + '/Instructions.xml';
    var destXMLFile = new File(destXMLFilename);
    if (!destXMLFile.open("w")) {
	return self.errorPrompt("Unable to open annotate XML file for writing: " + destXMLFile.error);
    }
    destXMLFile.close();
    destXMLFilename = decodeURI(destXMLFilename);

    // Try to load biglist
    try {
	var bigListFilename = parentFolder + '/etc/BigList.xml';
	var xml = loadBigList(bigListFilename);
	
    } catch (e) {
	var msg = ("Error loading " + bigListFilename + ':  ' + e);
	alert(msg);
	Error.runtimeError(9001, msg);
    }

    var featureXML = xml;
    ParamsResolver.featureList = featureXML;

    var annoXML = Stdlib.readXMLFile(sourceXMLFilename);

    //
    // loop through items in annoXML
    // for each item, find corresponding BigList feature

    // for each item
    //   create a set-operation (if not atomicSet)
    //   create a set-parameter (if params exist)
    //   create a commit


    var itemList = annoXML.xpath('//Item');
    var len = itemList.length();
    var instrList = new XML('<InstructionList/>');

    for (var i = 0; i < len; i++) {
	var item = itemList[i];
	var id = item.feature.@id.toString();
	var feature = featureXML.xpath('//feature[@id="' + id + '"]');

	if (feature == undefined) {
	    var msg = "Feature " + id + " not found!";
	    Stdlib.log(msg);
	    alert(msg);
	    continue;
	}

	if (feature.@id == "Unknown") {
	    var commitInstr = createUnknownInstr(item, feature);
	    instrList.appendChild(commitInstr);
	    continue;
	}

	if (feature.@type == "Ignore") {
	    continue;
	}

	ParamsResolver.getParams(feature);

	// create a set-operation (if not atomicSet)
	if (feature.@atomicSet != "Yes") {
	    var setOpInstr = createSetOpInstr(item, feature);
	    instrList.appendChild(setOpInstr);
	}

	// create a set-parameter (if params exist)
	var setParamsInstr = createSetParamsInstr(item, feature);
	if (setParamsInstr) {
	    instrList.appendChild(setParamsInstr);
	}

	// create a commit
	var commitInstr = createCommitInstr(item, feature, setParamsInstr);
	instrList.appendChild(commitInstr);
    }
    
    var xml = new XML('<Instructions/>');
    xml.Source = sourceXMLFilename;
    xml.Date = new Date().toString();
    
    //instrList.setLocalName('InstructionList');
    xml.appendChild(instrList);
    
    Stdlib.writeXMLFile(destXMLFilename, xml);
};


