#target photoshop

//
// GenerateSteps.jsx
//

var wordRegExp = /[\w\u00C0-\u00D6]/;
var stepNumRegExp = /step-(\d+)/;
var interactiveOn = true;
var parentFolderGlobal = '';
var destFolderGlobal = '';

function getStepNum(stepXML) {
    var stepNum = -1;
    if (stepXML) {
        var matches = stepXML.@id.match(stepNumRegExp);
        if (matches.length == 2) {
            stepNum = parseInt(matches[1]);
        }
    }

    return stepNum;
}

function execBlank(instr, blank) {
    var newText = instr.xpath(blank);
    var fn = blank.@fn.toString();
    // $.level = 1; debugger;

    // check if blank has modifiers
    if (fn) {
	newText = newText.toString();
	if (fn == "abs") {
	    newText = Math.abs(newText);

	} else if (fn == "strf") {
	    var fmt = blank.@fmt;
	    newText = File(newText).strf(fmt);

	} else {
	    Stdlib.log("Unknown format function: " + fn);
	}
    }

    return newText;
};

function applyTemplate(parent, textTempl, instr) {

    var text = textTempl.text.toString();
    var numBlanksFound = 0;
    var index = 0;
    var finalT = "";

    //
    // loop to find all instances of _blank_ in text
    //
    while (index < text.length) {

	// get appropriate substring to test
	var subText = text.substr(index);
	var matchBlank = new RegExp("_blank_", "gi");
	var blankPos = subText.search(matchBlank);

	// if found _blank_
	if (blankPos >= 0) {
	    // add preceding text to finalT
	    finalT += text.substr(index, blankPos);

	    // replace _blank_ with appropriate text
	    numBlanksFound +=1 ;
	    var blankNum = '_blank_'+numBlanksFound.toString();
	    var path = textTempl[blankNum];
	    var newText = execBlank(instr, path);
	    //var newText = instr.xpath(path);
	    newText = Common.localize(newText.toString());

	    // make first letter of description lower case
	    if ((path == "desc") && ((index+blankPos) > 0)) {
		//newText = newText.toLowerCase();
		if (newText.length > 0)
		    newText = newText[0].toLowerCase() +
			newText.slice(1, newText.length);
	    }

	    finalT += newText;

	    // increment substring index appropriately
	    index += blankPos + 7;
	}
	// if no _blank_ found, just add the rest of the
	// text to finalT and break out of loop
	else {
	    finalT += subText;
	    break;
	}
    }

    finalT = finalT.trim();
    var ch = (finalT.length >= 1) ? finalT[0] : '';

    if (ch != '<' && ch != '*') {
	// this converts the first ASCII word character to uppercase
	finalT = finalT.replace(wordRegExp, function(s) { return s.toUpperCase(); }); 
    }

    if (textTempl.@location == "Tool" && finalT.contains("[add icon]")) {
	var parts = finalT.split("[add icon]");
	parent.appendChild(parts[0]);
	var icon = createIconXML(instr);
	parent.appendChild(icon);
	parent.appendChild(parts[1]);

    } else if (finalT.match(/\[icon=.*\]/)) {
	var m = finalT.match(/\[icon=(.*)\]/);
	var iconName = m[1];

	var parts = finalT.split(m[0]);
	parent.appendChild(parts[0]);
	var icon = createIconXML(iconName);
	parent.appendChild(icon);
	parent.appendChild(parts[1]);

    } else {
	parent.appendChild(finalT);
    }

    return finalT;
};

function getTextForTemplate(parent, textTempl, instr) {
    var text = textTempl.text.toString();
    var numBlanksFound = 0;
    var index = 0;
    var finalT = "";

    //
    // loop to find all instances of _blank_ in text
    //
    while (index < text.length) {

	// get appropriate substring to test
	var subText = text.substr(index);
	var matchBlank = new RegExp("_blank_", "gi");
	var blankPos = subText.search(matchBlank);

	//finalT += "ST=[" + subText.toString() + "]";

	// if found _blank_
	if (blankPos >= 0) {
	    // add preceding text to finalT
	    finalT += text.substr(index, blankPos);

	    // replace _blank_ with appropriate text
	    numBlanksFound +=1 ;
	    var blankNum = '_blank_'+numBlanksFound.toString();
	    var path = textTempl[blankNum];

	    //var newText = instr.xpath(path);
	    var newText = execBlank(instr, path);
	    newText = Common.localize(newText.toString());

	    // make first letter of description lower case
	    if ((path == "desc") && ((index+blankPos) > 0)) {
		//newText = newText.toLowerCase();
		if (newText.length > 0)
		    newText = newText[0].toLowerCase() +
			newText.slice(1, newText.length);
	    }

	    finalT += newText;

	    // increment substring index appropriately
	    index += blankPos + 7;
	}
	// if no _blank_ found, just add the rest of the
	// text to finalT and break out of loop
	else {
	    finalT += subText;
	    break;
	}
    }

    finalT = finalT.trim();
    var ch = (finalT.length >= 1) ? finalT[0] : '';

    if (ch != '<' && ch != '*') {
	finalT = finalT.replace(wordRegExp, function(s) { return s.toUpperCase(); }); 
    }

    if (textTempl.@location == "Tool" && finalT.contains("[add icon]")) {
	var parts = finalT.split("[add icon]");
	parent.appendChild(parts[0]);
	var icon = createIconXML(instr);
	parent.appendChild(icon);
	parent.appendChild(parts[1]);

	// finalT = parts[0]+icon.toString()+parts[1];

    } else if (finalT.match(/\[icon=.*\]/)) {
	var m = finalT.match(/\[icon=(.*)\]/);
	var iconName = m[1];

	var parts = finalT.split(m[0]);
	parent.appendChild(parts[0]);
	var icon = createIconXML(iconName);
	parent.appendChild(icon);
	parent.appendChild(parts[1]);

    } else {
	parent.appendChild(finalT);
    }

    var str = parent.toXMLString();

    if (str.contains('&lt;') && str.contains('&gt;')) {
	str = str.replace(/&lt;/g, "<");
	str = str.replace(/&gt;/g, ">");
	var x = new XML(str);
	parent.setChildren(x.children());
    }

    return parent;
};

function createStepHeader(stepNum){
    var stepXML = new XML('<div class="step-container" />');
    stepXML.@id = 'step-' + stepNum;
    stepXML.@onclick = 'setCurrentStepFromDiv(this);';
    var temp = new XML('<span class="stepNum" />');
    temp.appendChild('Step ' + stepNum.toString() + ':');
    stepXML.appendChild(temp);
    var temp2 = new XML('<div class="step" />');
    stepXML.appendChild(temp2);
    var temp3 = new XML('<div class="stepText" />');
    temp2.appendChild(temp3);
    return [stepXML, temp2, temp3];
};

function createStepForCommitInstr(instr, templatesXML, stepNum, prevStepXML){
    var targetStepWidth = 900;
    var stepXML = null;
    var stepText = null;
    var usedPrevStepXML = false;    

    var instrId = instr.id;
    var instrLocation = instr.location;
    var instrDialog = instr.dialog;
    var instrRelatedUI = instr.relatedUI;

    // if prevStepXML is defined and is not a commit, just add text to it
    // otherwise, create new step    
    if (prevStepXML && getStepType(prevStepXML) != 'commit') {
	stepXML = prevStepXML;
	stepText = getStepText(prevStepXML);
	stepText.appendChild(' ');  // add space before new text
	usedPrevStepXML = true;
    }
    else {
	var stepHTML = createStepHeader(stepNum);
	stepXML = stepHTML[0];
	stepText = stepHTML[2];
    }

    // add additional info to step, including triggers
    if (stepXML) {
	stepXML.@hasdialog = (instrDialog == 'Yes') ? "true" : "false";
	stepXML.@type = "commit";
	// add eventtrigger, padded to 4 chars
	var eventString = instr.event.toString();
    if (eventString.length <= 4) {
        stepXML.@eventtrigger = eventString + Array(5-eventString.length).join(' ');
        }
    if (instrId == 'File-Open') {
        stepXML.@historytrigger = 'Open|openDocNoHistoryBug';
        }
    }

    // get possible text template
    var x = "//template[@class='commit']";
    var possibleTempl = templatesXML.xpath(x);
    var textTempl = null;

    // search based on instrId, instrLocation and instrDialog
    for (var j = 0; j < possibleTempl.length(); j++){
	var tempLocation = possibleTempl[j].@location;
	var tempDialog = possibleTempl[j].@dialog;
	if (instrId.indexOf(possibleTempl[j].@id) > 0) {
	    textTempl = possibleTempl[j];
	    break;
	}else if ( tempLocation == instrLocation && tempDialog == instrDialog){
	    textTempl = possibleTempl[j];
	    break;
	}
    }

    // check whether valid text template has been found and apply
    var finalT = "";
    if (textTempl != null){
	finalT = applyTemplate(stepText, textTempl, instr);
    } 

    // add code to step
    var codeStepXML = stepXML;

    // adding original jsx code for HTML so that tutorials can be interactive
    var codeHere = instr.literal;
    if (codeHere && interactiveOn && codeStepXML){

	var codeId = "tbstep-"+getStepNum(codeStepXML);
        var instrId = codeStepXML.@psid.toString();
        // if file open, add assets attribute
        var isFileOpen = (instrId == "File-Open" || instrId == "File-OpenAsSmartObject");
        var stepCodeStr = "";
        if (isFileOpen) 
	    {
            var cmd = instr.literal.toString();
            var m = cmd.match(/new File\( "(.*)" \)/);
            var parts = m[1].split('/');
            if (parts.length < 2) 
                parts = m[1].split('\\');
            var fileName = parts.pop();
            if (!fileName.endsWith('.jpg'))
                fileName += '.jpg';

            stepCodeStr = '<script id="' + codeId + '" asset="assets/' + fileName + '" />';
	    }
        else {
            stepCodeStr = '<script id="'+codeId+'" />';
        }

        var stepCode = new XML(stepCodeStr);
        stepCode.appendChild(instr.literal);
        
        codeStepXML.insertChildBefore(codeStepXML.child(0), stepCode);

	// get "step" div in codeStepXML
	var stepDivs = codeStepXML.child("div");
	if (stepDivs.length() >= 1)
	{
	    stepDivs[0].appendChild(new XML('<div class="stepLink"><a class="showme" href="'+codeId+'"><span></span></a></div>'));
	}
	
    }

    if (usedPrevStepXML) 
	return null;
    else
	return stepXML;
};

function createStepForSetOpInstr(instr, templatesXML, stepNum, blocks){
    var targetStepWidth = 900;

    var stepHTML = createStepHeader(stepNum);
    var stepXML = stepHTML[0];
    var stepDiv = stepHTML[1];
    var stepText = stepHTML[2];

    var instrLocation = instr.location;
    var x = "//template[@class='set-operation']";
    var possibleTempl = templatesXML.xpath(x);
    var textTempl = null;

    for (var j = 0; j < possibleTempl.length(); j++){
	var tempLocation = possibleTempl[j].@location;
	if ( tempLocation == instrLocation){
	    textTempl = possibleTempl[j];
	    break;
	}
    }

    var finalT = "";
    if (textTempl != null){
	finalT = applyTemplate(stepText, textTempl, instr);
    } else {
	//if there is no template then a set operation is not included in the tutorial.
	stepXML = null;
    }

    if (stepXML) {
	stepXML.@type = "set-op";
	stepXML.@psid = instr.id;
    }

    return stepXML;
};

function createSetParameterText(parent, cparam, templates, depth) {
    if (cparam.toXMLString().contains("$$$/TB/")) {
	// $.level = 1; debugger;
    }
    var paramWidget = cparam.widget;
    var paramValue = Common.localize(cparam.value.toString());
    var x = "//template[@class='set-params']";
    var possibleTempl = templates.xpath(x);
    var textTempl = undefined;

    for (var j = 0; j<possibleTempl.length(); j++){
	var tempWidget = possibleTempl[j].@widget;
	if (tempWidget == paramWidget){
	    var tempValue = possibleTempl[j].@value;
	    if (tempValue != ""){
		if(tempValue == paramValue){
		    textTempl = possibleTempl[j];
		    break;
		}
	    }else{
		textTempl = possibleTempl[j];
		break;
	    }
	}
    }

    if (textTempl != null) {
	getTextForTemplate(parent, textTempl, cparam);

    } else {
	var str = paramWidget ? ('(' + paramWidget + ')') : '';
	str = Common.localize(cparam.name.toString()) + ' ' + str;
	parent.appendChild("**missing a parameter description for " + str + ".**");
    }

    //recurse into createSetParameterText
    if (cparam.params != undefined && cparam.params['param'].length() > 0 && cparam.widgettype != "approximate"){
	var bl = new XML("<ul />");
	createSetParameterTextForParamsBlock(cparam.params, templates, depth+1, bl);
	parent.appendChild(bl);

	/*
	  for (var i=0; i<cparam.params['param'].length(); i++){
	  var newParam = cparam.params['param'][i];
	  createSetParameterText(parent, newParam, templates, depth+1);
	  }
	*/
    }

    return parent;
};

function createSetParameterTextForParamsBlock(params, templates, depth, stepText) {
    // Create bullets if there is a params list or if the recursion depth is > 0
    var paramList = (params['param'].length() > 1 || depth > 0);
    if (params['param'].length() > 1 && depth == 0){
	//paramList = true;
	//create bulleted list

	stepText.appendChild("Change the following settings:");
	var bl = new XML("<ul />");
	stepText.appendChild(bl);
	stepText = bl;
    }

    for (var i = 0; i< params['param'].length(); i++){
	var cparam = params['param'][i];
	var tmp = <tmp/>;
	createSetParameterText(tmp, cparam, templates, depth);

	var children = tmp.children();
	var len = children.length();

	if (paramList){
	    var li = new XML("<li />");
	    for (var j = 0; j < len; j++) {
		li.appendChild(children[j]);
	    }

	    stepText.appendChild(li);

	} else {
	    for (var j = 0; j < len; j++) {
		stepText.appendChild(children[j]);
	    }
	}
    }
};

function getStepText(stepXML) {
    var stepText = null;
    if (stepXML) {
	var divs1 = stepXML.child('div');
	for (var i = 0; i < divs1.length(); i++) {
	    if (divs1[i].attribute('class') == 'step') {
		var divs2 = divs1[i].child('div');
		for (var j = 0; j < divs2.length(); j++) {
		    if (divs2[j].attribute('class') == 'stepText') {
			stepText = divs2[j];
		    }
		}
	    }
	}
    }

    return stepText;
}

function getStepType(stepXML) {
    var stepType = 'undefined';
    if (stepXML) {
	stepType = stepXML.attribute('type');
    }

    return stepType;
}

function createStepForSetParamInstr(instr, templates, stepNum, prevStepXML){
    var params = instr.params;
    var stepXML = null;
    var stepText = null;
    var usedPrevStepXML = false;

    // if prevStepXML is defined and is a set-operation, just add text to it
    // otherwise, create new step    
    if (prevStepXML && getStepType(prevStepXML) == 'set-op') {
	stepXML = prevStepXML;
	stepText = getStepText(prevStepXML);
	stepText.appendChild(' ');  // add space before new text
	usedPrevStepXML = true;
    }
    else {
	var stepHTML = createStepHeader(stepNum);
	stepXML = stepHTML[0];
	stepText = stepHTML[2];
    }

    createSetParameterTextForParamsBlock(params, templates, 0, stepText);

    stepXML.@type = "set-params";
    stepXML.@psid = instr.id;

    if (usedPrevStepXML)
	return null;
    else
	return stepXML;
};

function createHeadHTML(title) {
    var head = new XML('<head />');
    head.appendChild(new XML('<title>'+title+'</title>'));
    head.appendChild(<meta http-equiv="Content-Type" content="text/html;charset=UTF-8" />);

    head.appendChild(new XML('<style type="text/css">@import url("main.css");</style>'));
    head.appendChild(new XML('<replaceWithIESpecificCode />'));
    head.appendChild(new XML('<link rel="stylesheet" type="text/css" href="js/fancybox/jquery.fancybox-1.3.0.css" media="screen" />'));
    head.appendChild(new XML('<link rel="stylesheet" type="text/css" href="js/jHtmlArea/style/jHtmlArea.css" media="screen" />'));
    head.appendChild(new XML('<script language="JavaScript" type="text/javascript" src="js/jquery.min.js">;</script>'));
    head.appendChild(new XML('<script language="JavaScript" type="text/javascript" src="js/jquery.scrollto.js">;</script>'));
    head.appendChild(new XML('<script language="JavaScript" type="text/javascript" src="js/fancybox/jquery.easing-1.3.pack.js">;</script>'));
    head.appendChild(new XML('<script language="JavaScript" type="text/javascript" src="js/fancybox/jquery.mousewheel-3.0.2.pack.js">; </script>'));
    head.appendChild(new XML('<script language="JavaScript" type="text/javascript" src="js/fancybox/jquery.fancybox-1.3.0.pack.js">;</script>'));
    head.appendChild(new XML('<script language="JavaScript" type="text/javascript" src="js/jHtmlArea/scripts/jHtmlArea-0.7.5.min.js">;</script>'));
    head.appendChild(new XML('<script language="JavaScript" type="text/javascript" src="js/main.js">  ;</script>'));
    head.appendChild(new XML('<script language="JavaScript" type="text/javascript" src="js/editing.js">  ;</script>'));
    head.appendChild(new XML('<script language="JavaScript" type="text/javascript" src="js/logging.js">  ;</script>'));
    return head;
};

function createDialogHTML() {
    var dialogContainerXML = new XML('<div id="dialog-container" />');
    var dialogXML = new XML('<div id="dialog" />');
    dialogXML.appendChild(new XML('<div id="dialog-title">Photoshop needs your attention</div>'));
    dialogXML.appendChild(new XML('<div id="dialog-message">After you complete this step, dismiss the dialog box in Photoshop to move on to the next step.</div>'));
    var dialogStepXML = new XML('<div id="dialog-step" />');
    dialogStepXML.appendChild(new XML('<span id="dialog-stepNum">Replace with step num</span>'));
    dialogStepXML.appendChild(new XML('<div id="dialog-stepText">Replace with step text</div>'));
    dialogXML.appendChild(dialogStepXML);
    dialogXML.appendChild(new XML('<div id="dialog-reset">If you believe Tutorial Player is wrong and there is no open dialog box in Photoshop, please restart this tutorial by hitting the BACK button in the app title bar above and then selecting the tutorial again from the gallery page.  If that still does not resolve the problem, you may need to restart Photoshop and Tutorial Player.</div>'));
    dialogContainerXML.appendChild(dialogXML);
    return dialogContainerXML;
}

function addLoggingHTML(body) {
    if (!body) {
        return;
    }
    var script = new XML('<script language="JavaScript" type="text/javascript" src="js/s_code.js"></script>');
    script.appendChild(' ');
    body.appendChild(script);
};

function createBeforeAfterHTML(destFolder){

    var beforeAfter = '';
   // Add before/after images if necessary
    var beforeImg = new File(destFolder + '/html/images/before.png');    
    var afterImg = new File(destFolder + '/html/images/after.png');    
    
    if (beforeImg.exists || afterImg.exists) {
	 beforeAfter = new XML('<div id="beforeAfter" />');
	if (beforeImg.exists) {
        
	    var before = new XML('<div id="before" />');
	    before.appendChild(new XML('<a href="images/before.png" ><img id="beforeImg" src="images/before.png" /></a>'));    
	    before.appendChild(new XML('<div class="beforeAfterCaption">Before</div>'));
	    beforeAfter.appendChild(before);
	}
	if (afterImg.exists) {
	    var after = new XML('<div id="after" />');
	    after.appendChild(new XML('<a href="images/after.png"><img id="afterImg" src="images/after.png" /></a>'));
	    after.appendChild(new XML('<div class="beforeAfterCaption">After</div>'));
	    beforeAfter.appendChild(after);
	}
    }

    return beforeAfter;

};

function createHeaderHTML(title, author, description, destFolder) {

    var header = new XML('<div id="header" />');
    header.appendChild(new XML('<div class="title">'+title+'</div>'));
    header.appendChild(new XML('<div class="author">by '+author+'</div>'));

    if (description.length > 0) {
	var desc = new XML('<div class="description"><span id="goal">Goal:</span><div id="text" class="description-text" style="width:512px;">' + description + '</div></div>');
	header.appendChild(desc);
    }

    return header;
    
}


function createIconXML(instr) {
    var icon = new XML('<img/>');
    var iconName;
    if (instr.constructor == String) {
	iconName = instr;

    } else {
	iconName = instr.id.toString().replace(/Tool\w*/, '');
    }

    var iconFile = Common.resolveIconName(iconName);

    if (iconFile.exists) {
	var dicon = new File(destFolderGlobal + '/html/icons/' + iconFile.name);
	if (!dicon.exists) {
	    iconFile.copy(dicon);
	}
	if (!dicon.exists) {
	    Stdlib.log("Unable to copy icon file: " + dicon.toUIString());
	}

    } else {
	Stdlib.log("Unable to find icon file: " + iconFile.toUIString());
    }

    icon.@src = "icons/" + decodeURI(iconFile.name);
    icon.@alt=  "[" + iconName + " Icon]";

    return icon;
};

function addImage(div, img, destFolder) {
    var wrapXml = new XML('<div class="img" />');
    var imgXml = new XML('<img/>');
    var fname = img.source.@path.toString();
    var origImageFolder = new File(GenerateStepsUI.opts.source).parent;
    var file = new File(origImageFolder + '/' + fname);
    if (file.exists) {
	var newFile = new File(destFolder + '/images/' + file.name);
	if (file.copy(newFile)) {
	    fname = newFile.parent.name + '/' + newFile.name;
	} else {
	    Error.runtimeError(Stdlib.IO_ERROR_CODE, fileErrorMessage(file));
	}
    }
    imgXml.@src = fname;
    imgXml.@id = "img" + imageCounter.toString();
    imageCounter++;
    wrapXml.appendChild(imgXml);
    div.appendChild(wrapXml);
};

// WL: resize images in div to fit target width
function resizeImages(div, targetStepWidth, destFolder) {

    var divXml = new XML(div.toString());
    var imgs = divXml.xpath("//img");
    var totalWidth = 0;
    for (var i = 0; i < imgs.length(); i++) {
	var matchImgDims = new RegExp("([0-9]+)x([0-9]+)\.png", "gi");
	var match = matchImgDims.exec(imgs[i].@src);
	if (match) {
	    var imgWidth = parseInt(match[0]);
	    totalWidth += imgWidth;
	}
    }

    if (totalWidth > 0) {
	var wraps = divXml.xpath("//*[@class='img']");
	for (var i = 0; i < wraps.length(); i++) {
	    var wrapXml = new XML(wraps[i].toString());
	    var wrappedImg = wrapXml.xpath("//img");
	    if (wrappedImg != undefined)  {
		var matchImgDims = new RegExp("([0-9]+)x([0-9]+)\.png", "gi");
		var match = matchImgDims.exec(wrappedImg.@src);
		if (match) {
		    var imgWidth = parseInt(match[0]);
		    var newImgWidth = (imgWidth/totalWidth) * 97;
		    var newImgWidthPx = (imgWidth/totalWidth) * targetStepWidth - 25;
		    wraps[i].@style = 'float:left;padding:5px;';

		    // update CSS file
		    var cssFile = new File(destFolder + '/html/main.css');
		    if (cssFile.exists) {
			cssFile.encoding = "UTF8";
			cssFile.open('a');
			cssFile.write("#" + wrappedImg.@id + " {\n");
			cssFile.write("\tmax-width:" + newImgWidthPx + "px;\n");
			cssFile.write("\twidth:expression( this.width > " + newImgWidthPx + " ? " + newImgWidthPx + " : true );\n");
			cssFile.write("}\n");
			cssFile.close();
		    }

		}
	    }
	}
    }

    return divXml;
};

function generateSteps(srcXML, templatesXML){
    var instrList = srcXML.xpath('//Instruction');
    var len = instrList.length();

    var stepsList = new XML('<div id="steps"/>');
    var prevStepXML = null;

    var stepNum = 1;

    for (var i = 0; i < len; i++) {
	var instruction = instrList[i];
	var type = instruction.type;

	// get blocks of screenshots from
	// corresponding commit instruction
	// (WL: should maybe just process instructions by
	// groups based on commits to avoid doing this repeatedly)
	//MD: this code is not being used right now - 12-14-09
	var blocks = null;
	for (var j = i; j < len; j++) {
	    var nextInstr = instrList[j];
	    if (nextInstr.id != instruction.id) {
		break;
	    } else if (nextInstr.type == "commit") {
		var nextInstrXML = new XML(nextInstr.toString());
		blocks = nextInstrXML.xpath("//Block");
		break;
	    }
	}

	Stdlib.log(instruction.id);

	if (instruction.id == "Edit-Fill") {
	    // $.level = 1; debugger;
	}

	var stepXML = null;

	if (type == 'set-operation') {
	    stepXML = createStepForSetOpInstr(instruction, templatesXML, stepNum, blocks);
	} else if (type == 'set-params') {
	    stepXML = createStepForSetParamInstr(instruction, templatesXML, stepNum, prevStepXML);
	} else if (type == 'commit'){
	    stepXML = createStepForCommitInstr(instruction, templatesXML, stepNum, prevStepXML);
	}

	if (stepXML != null){
	    stepNum++;
	    stepsList.appendChild(stepXML);
	    prevStepXML = stepXML;
	}
    }

    generateFeedbackDiv(stepsList);

    return stepsList;
};

function generateFeedbackDiv(stepsList) {

    if (!stepsList)
	return;
    
    var stepXML = new XML('<div class="step-container" />');
    var feedbackContainerXML = new XML('<div id="feedback-container" />');
    var feedbackXML = new XML('<div id="feedback" />');
    var stepLinkXML = new XML('<div class="stepLink" />');
    var totopXML = new XML('<div id="totop" onclick="initTutorialState();updateCurrentStep();" />');
    var emptySpanXML = new XML('<span/>');
    totopXML.appendChild(emptySpanXML);
    stepLinkXML.appendChild(totopXML);
    feedbackXML.appendChild(" ");
    feedbackContainerXML.appendChild(feedbackXML);
    feedbackContainerXML.appendChild(stepLinkXML);
    stepXML.appendChild(feedbackContainerXML);

    stepsList.appendChild(stepXML);
}


function copyFiles(src, target, recursive) {
    Stdlib.log("\r\nCopying files from " + decodeURI(src) + " to " +
	       decodeURI(target));

    var files = src.getFiles();
    for (var i = 0; i < files.length; i++) {
	var file = files[i];

	if (file instanceof Folder) {
	    if (recursive) {
		var f = new Folder(target + '/' + file.name);
		f.create();
		if (!copyFiles(file, f, recursive)) {
		    return false;
		}
	    }
	    continue;
	}
	if (!copyFile(file, target)) {
	    return false;
	}
    }
    return true;
};


function copyFile(srcFile, target, flatten) {

    if (!(target instanceof Folder)) {
	Error.runtimeError(9002, "Internal Error: copyFile(" +
	                   srcFile + ", " + target + ")");
    }

    if (!target.exists) {
	Error.runtimeError(9002, "The folder " + decodeURI(target.fsName) +
	                   " does not exist.");
    }

    Stdlib.log("Copy file from " + decodeURI(srcFile) + " to " +
	       decodeURI(target));
    var file = new File(target + "/" + srcFile.name);

    if (flatten) {
	var flat = new Flattener();

	flat.exec(srcFile, file);

    } else {
	if (!srcFile || !srcFile.copy) {
	    // $.level = 1; debugger;
	}
	if (!srcFile.copy(file)) {
	    var msg = "Copy failed: " + srcFile.error + "\r\n" + decodeURI(srcFile);
	    Stdlib.log(msg);
	    return false;
	}
    }

    return true;
};

function GenerateSteps_generateSteps(parentFolder, destFolder, title, caption, author, description) {

    parentFolderGlobal = parentFolder;
    destFolderGlobal = destFolder;

    // Create HTML folder
    var htmlFolder = new Folder(destFolder + '/html');
    htmlFolder.create();

    // Check that destination HTML file location is valid
    var destHTMLFilename = destFolder + '/html/' + title + '.html';
    var destHTMLFile = new File(destHTMLFilename);
    if (!destHTMLFile.open("w")) {
	return self.errorPrompt("Unable to open html file for writing: " + destHTMLFile.error);
    }
    destHTMLFile.close();
    destHTMLFilename = decodeURI(destHTMLFilename);

    // Create icons folder
    var iconFolder = new Folder(destFolder + '/html/icons');
    iconFolder.create();

    // Copy CSS files
    var cssFile = new File(parentFolderGlobal + '/etc/main.css');
    if (cssFile.exists) {
	var newFile = new File(destFolder + '/html/main.css');
	cssFile.copy(newFile);
    }
    var cssIEFile = new File(parentFolderGlobal + '/etc/main_ie.css');
    if (cssIEFile.exists) {
	var newFile = new File(destFolder + '/html/main_ie.css');
	cssIEFile.copy(newFile);
    }

    // Copy over JS and images for CSS from etc
    var jsFolder = new Folder(parentFolderGlobal + '/etc/js');
    var tartgetJSFolder = new Folder(destFolder + '/html/js');
    tartgetJSFolder.create();
    copyFiles(jsFolder, tartgetJSFolder, true);
    var imagesFolder = new Folder(parentFolderGlobal + '/etc/images');
    var targetImagesFolder = new Folder(destFolder +'/html/images');
    targetImagesFolder.create();
    copyFiles(imagesFolder, targetImagesFolder, true);

    // Copy before/after images if necessary
    var beforeImg = new File(destFolder + '/before.png');
    if (beforeImg.exists)
	beforeImg.copy(new File(destFolder + '/html/images/before.png'));
    var afterImg = new File(destFolder + '/after.png');
    if (afterImg.exists)
	afterImg.copy(new File(destFolder +
    '/html/images/after.png'));    

    // Grab source Instructions and textTemplates XML files
    var sourceXMLFilename = destFolder + '/Instructions.xml';
    var templatesXMLFilename = parentFolderGlobal + '/etc/textTemplates.xml';
    var sourceXML = Stdlib.readXMLFile(sourceXMLFilename);
    var templatesXML = Stdlib.readXMLFile(templatesXMLFilename);
    var stepsList = generateSteps(sourceXML, templatesXML);

    // Write out HTML
    var htmlOut = new XML('<html editable="true"/>');
    // WL: removing for now since it shows up in the resulting page
    //htmlOut.Source = File(opts.source).toUIString();
    //htmlOut.Date = new Date().toString();
    var headXml = createHeadHTML(title);
    htmlOut.appendChild(headXml);
    var body = new XML('<body />');
    
    addLoggingHTML(body);
    //alert(body);
    
    var beforeAfterHTML = createBeforeAfterHTML(destFolder);
    body.appendChild(beforeAfterHTML);
    var tutText = new XML('<div id="tutorial-text" />');
    body.appendChild(tutText);
    
    var debugElement = new XML('<div id="debug" style="padding-left:500px;display:none;">DEBUG:</div>');      
    body.appendChild(debugElement);
    
    var header = createHeaderHTML(title, author, description, destFolder);
    tutText.appendChild(header);
    
    tutText.appendChild(stepsList);

    var dialogContainerXML = createDialogHTML();
    body.appendChild(dialogContainerXML);

    htmlOut.appendChild(body);

    Stdlib.writeXMLFile(destHTMLFilename, htmlOut);

    // WL: Add code to include IE-specific stylesheet
    if (!destHTMLFile.open("e")) {
	return self.errorPrompt("Unable to open html file for editing: " + destHTMLFile.error);
    }
    var fileString = destHTMLFile.read();
    var newFileString = fileString.replace('<replaceWithIESpecificCode/>', '<!--[if gte IE 6]><link rel="stylesheet" type="text/css" href="main_ie.css" /><![endif]-->');
    destHTMLFile.remove();
    destHTMLFile.close();

    var destHTMLFile = new File(destHTMLFilename);
    if (!destHTMLFile.open("w")) {
	return self.errorPrompt("Unable to open html file for writing: " + destHTMLFile.error);
    }
    destHTMLFile.write(newFileString);
    destHTMLFile.close();
};

"GenerateStepsClean.jsx";
// EOF


